package at.bestsolution.rsd.rt.server.jetty;

import java.util.Arrays;

import javax.websocket.server.ServerContainer;
import javax.websocket.server.ServerEndpoint;
import javax.websocket.server.ServerEndpointConfig;
import javax.websocket.server.ServerEndpointConfig.Configurator;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.websocket.jsr356.server.deploy.WebSocketServerContainerInitializer;

public class JettyWSServer {
	private Server server;

	public JettyWSServer(Configurator configurator, int port, Class<?>... endpointClasses) {
		server = new Server();
        ServerConnector connector = new ServerConnector(server);
        connector.setPort(port);
        server.addConnector(connector);
        
        ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
        context.setContextPath("/");
        server.setHandler(context);

        try
        {
            // Initialize javax.websocket layer
            ServerContainer wscontainer = WebSocketServerContainerInitializer.configureContext(context);
            for( Class<?> cl : endpointClasses ) {
            	wscontainer.addEndpoint(createEndpointConfig(cl, configurator));	
            }
        }
        catch (Exception t)
        {
            t.printStackTrace(System.err);
        }
	}
	
	 private static ServerEndpointConfig createEndpointConfig(Class<?> endpointClass, Configurator configurator) {
	    	ServerEndpoint annotation = endpointClass.getAnnotation(ServerEndpoint.class);
	    	return ServerEndpointConfig.Builder.create(endpointClass, annotation.value())
	                .subprotocols(Arrays.asList(annotation.subprotocols()))
	                .decoders(Arrays.asList(annotation.decoders()))
	                .encoders(Arrays.asList(annotation.encoders()))
	                .configurator(configurator)
	                .build();
	    }
	
	public void run() {
		try {
			server.start();
			server.dump(System.err);
	        server.join();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
	}
}
